"""
PyXLL Examples: Real time data

As well as returning static values from functions, PyXLL functions
can return special 'RTD' instances that can notify Excel of
updates to their value.

This could be used for any real time data feed, such as live
prices or the status of a service.
"""
from pyxll import RTD, xl_func, xl_app
from datetime import datetime
import threading
import logging
import time
import random
import json
import urllib.request, urllib.error, urllib.parse

_log = logging.getLogger(__name__)

class CoinPriceRTD(RTD):

    def __init__(self, symbol):
        initial_value = 0
        super(CoinPriceRTD, self).__init__(value=initial_value)
        self.__symbol = symbol
        self.__running = True
        self.__thread = threading.Thread(target=self.__thread_func)
        self.__thread.start()

    def connect(self):
        # Called when Excel connects to this RTD instance, which occurs
        # shortly after an Excel function has returned an RTD object.
        _log.info("CoinPriceRTD Connected")

    def disconnect(self):
        # Called when Excel no longer needs the RTD instance. This is
        # usually because there are no longer any cells that need it
        # or because Excel is shutting down.
        self.__running = False
        _log.info("CoinPriceRTD Disconnected")

    def __thread_func(self):
        while self.__running:
            # Setting 'value' on an RTD instance triggers an update in Excel
            try:
                opener = urllib.request.build_opener()
                opener.addheaders = [('User-Agent', 'Mozilla/5.0')]
                opener.addheaders = [('Cache-Control', 'max-age=0')]
                url = "https://coincap.io/page/%s" % self.__symbol
                response = opener.open(url)
                data = response.read()
                data = data.decode()

                # the returned data is in json format
                result = json.loads(data).get("price", "#NoLatestPrice")
            except Exception as e:
                result = e
            new_value = result
            if self.value != new_value:
                self.value = new_value
            time.sleep(3)

@xl_func("string symbol: rtd")
def coin_price_rtd(symbol):
    return CoinPriceRTD(symbol)

